//  Diffusion Client Library for iOS, tvOS and OS X / macOS
//
//  Copyright (c) 2015, 2017 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;
#import <Diffusion/PTDiffusionMetadata.h>

@class PTDiffusionFieldMetadata;

NS_ASSUME_NONNULL_BEGIN

/**
 @brief Record metadata describes a data item that can represent a collection of fields.

 A record metadata object is immutable.

 Default multiplicity is defined by the `defaltMultiplicity` class method on PTDiffusionMultiplicity.

 @deprecated Since 6.0. Use PTDiffusionRecordV2 or PTDiffusionJSON.

 @since 5.6
 */
__deprecated_msg("Will be removed in a future release.")
@interface PTDiffusionRecordMetadata : PTDiffusionMetadata

/**
 Returns a record metadata object initialized with the given name, default multiplicity
 and given field metadata array.

 @param name The name to use. It is used when looking up records that have been
 transformed using the receiver.

 @param fieldMetadata The field metadata defining what fields are expected as well as
 what order they should appear in. This array may not be empty.

 @exception NSInvalidArgumentException If any of the given arguments are `nil` or
 the field metadata array is empty.

 @since 5.6
 */
-(instancetype)initWithName:(NSString *)name
              fieldMetadata:(NSArray<__kindof PTDiffusionFieldMetadata *> *)fieldMetadata;

/**
 Returns a record metadata object initialized with the given name, given multiplicity
 and given field metadata array.

 @param name The name to use. It is used when looking up records that have been
 transformed using the receiver.

 @param multiplicity The multiplicity to use. It governs how many records
 can be transformed using the receiver from the enclosing record content.

 @param fieldMetadata The field metadata defining what fields are expected as well as
 what order they should appear in. This array may not be empty.

 @exception NSInvalidArgumentException If any of the given arguments are `nil` or
 the field metadata array is empty.

 @since 5.6
 */
-(instancetype)initWithName:(NSString *)name
               multiplicity:(PTDiffusionMultiplicity *)multiplicity
              fieldMetadata:(NSArray<__kindof PTDiffusionFieldMetadata *> *)fieldMetadata NS_DESIGNATED_INITIALIZER;

/**
 The field metadata used to transform records using the receiver. This array may not be empty.

 @since 5.6
 */
@property(nonatomic, readonly, copy) NSArray<__kindof PTDiffusionFieldMetadata *> * fieldMetadata;

/**
 Compares the receiver to the given record metadata.

 @param recordMetadata The record metadata object with which to compare the receiver.

 @return `YES` if the record metadata is equal to the receiver, otherwise `NO`.

 @since 5.6
 */
-(BOOL)isEqualToRecordMetadata:(nullable PTDiffusionRecordMetadata *)recordMetadata;

@end

NS_ASSUME_NONNULL_END
