//  Diffusion Client Library for iOS, tvOS and OS X / macOS
//
//  Copyright (c) 2015, 2017 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;
#import <Diffusion/PTDiffusionFieldMetadata.h>

NS_ASSUME_NONNULL_BEGIN

/**
 @brief String field metadata describes a data item that can have a textual value.

 A string field metadata object is immutable.
 
 Default multiplicity is defined by the `defaltMultiplicity` class method on PTDiffusionMultiplicity.
 
 @deprecated Since 6.0. Use PTDiffusionRecordV2 or PTDiffusionJSON.

 @since 5.6
 */
__deprecated_msg("Will be removed in a future release.")
@interface PTDiffusionStringFieldMetadata : PTDiffusionFieldMetadata

/**
 The default value used if one is not explicitly specified on initialisation.

 @return An empty string.

 @since 5.7.1
 */
+(NSString *)defaultDefaultValue;

/**
 Returns a string field metadata object initialized with the given name and multiplicity.

 @param name The name to use. It is used when looking up fields that have been
 transformed using the receiver.

 @param multiplicity The multiplicity to use. It governs how many fields
 can be transformed using the receiver from the enclosing record.

 @exception NSInvalidArgumentException If either the given name or multiplicity is `nil`.

 @since 5.6
 */
-(instancetype)initWithName:(NSString *)name
               multiplicity:(PTDiffusionMultiplicity *)multiplicity;

/**
 Returns a string field metadata object initialized with the given name, given default
 value and given multiplicity.

 @param name The name to use. It is used when looking up fields that have been
 transformed using the receiver.

 @param defaultValue The default value that required occurrences of the field should
 be initialized to. Specifying `nil` for this argument uses an empty string
 (as specified by the defaultDefaultValue class method).

 @param multiplicity The multiplicity to use. It governs how many fields
 can be transformed using the receiver from the enclosing record.

 @exception NSInvalidArgumentException If either the given name or multiplicity is `nil`.

 @since 5.7.1
 */
-(instancetype)initWithName:(NSString *)name
               defaultValue:(nullable NSString *)defaultValue
               multiplicity:(PTDiffusionMultiplicity *)multiplicity NS_DESIGNATED_INITIALIZER;

/**
 The default value that required occurrences of the field should be initialized to.

 @since 5.7.1
 */
@property(nonatomic, readonly) NSString* defaultValue;

/**
 Compares the receiver to the given string field metadata.

 @param stringFieldMetadata The string field metadata object with which to compare the receiver.

 @return `YES` if the string field metadata is equal to the receiver, otherwise `NO`.

 @since 5.6
 */
-(BOOL)isEqualToStringFieldMetadata:(nullable PTDiffusionStringFieldMetadata *)stringFieldMetadata;

@end

NS_ASSUME_NONNULL_END
