//  Diffusion Client Library for iOS, tvOS and OS X / macOS
//
//  Copyright (c) 2016, 2017 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;

NS_ASSUME_NONNULL_BEGIN

/**
 @brief Attributes applicable to all topic types.
 
 A topic attributes object is immutable.
 
 @see PTDiffusionTopicDetailsBuilder

 @deprecated Since 6.0. The use of PTDiffusionTopicDetails is being phased out
 in favor of PTDiffusionTopicSpecification. This interface will be removed in a
 future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
@interface PTDiffusionTopicAttributes : NSObject <NSCopying>

+(instancetype)new NS_UNAVAILABLE;

-(instancetype)init NS_UNAVAILABLE;

/**
 The topic reference or a zero length string if not set.

 @since 5.7
 */
@property(nonatomic, readonly) NSString* reference;

/**
 The 'tidy on unsubscribe' flag.
 
 By default, if a client unsubscribes from a topic, it may still receive
 content already queued for it on that topic.
 
 If this option is set to `YES`, when a client unsubscribes from a topic then
 any content for that topic that is still queued for the client is removed.
 There can be a performance overhead to using this option as the client queue
 must be locked whilst content is removed, however it may prove useful for 
 preventing unwanted data being sent to clients.

 @since 5.7
 */
@property(nonatomic, readonly) BOOL tidiesOnUnsubscribe;

/**
 Additional properties.

 @since 5.7
 */
@property(nonatomic, readonly) NSDictionary<NSString *, NSString *>* properties;

/**
 Compares the receiver to the given topic attributes.

 @param topicAttributes The topic attributes object with which to compare the receiver.

 @return `YES` if the topic attributes is equal to the receiver, otherwise `NO`.

 @since 5.7
 */
-(BOOL)isEqualToTopicAttributes:(nullable PTDiffusionTopicAttributes *)topicAttributes;

@end

NS_ASSUME_NONNULL_END
