//  Diffusion Client Library for iOS, tvOS and OS X / macOS
//
//  Copyright (c) 2015, 2017 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;
#import <Diffusion/PTDiffusionTopicType.h>

@class PTDiffusionFieldMetadata;
@class PTDiffusionRecordContentSchema;
@class PTDiffusionRecordMetadata;
@class PTDiffusionTopicAttributes;
@class PTDiffusionTopicSchema;

@protocol PTDiffusionRecordTopicDetailsBuilder;
@protocol PTDiffusionRoutingTopicDetailsBuilder;
@protocol PTDiffusionSingleValueTopicDetailsBuilder;
@protocol PTDiffusionSlaveTopicDetailsBuilder;
@protocol PTDiffusionTopicDetailsBuilder;

NS_ASSUME_NONNULL_BEGIN

/**
 Name of exception raised when topic details cannot be created due to incomplete or incorrect
 builder configuration.

 @deprecated Since 6.0. The use of PTDiffusionTopicDetails is being phased out
 in favor of PTDiffusionTopicSpecification. This interface will be removed in a
 future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
extern NSString *const PTDiffusionInvalidTopicDetailsBuilderStateException;

/**
 Name of exception raised when topic details cannot be created due to lack of support in
 this client library for the topic type.

 @deprecated Since 6.0. The use of PTDiffusionTopicDetails is being phased out
 in favor of PTDiffusionTopicSpecification. This interface will be removed in a
 future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
extern NSString *const PTDiffusionUnsupportedTopicDetailsBuildException;

/**
 The signature required for a builder block accepting a record topic details builder.

 @deprecated Since 6.0. The use of PTDiffusionTopicDetails is being phased out
 in favor of PTDiffusionTopicSpecification. This interface will be removed in a
 future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
typedef void (^PTDiffusionRecordTopicDetailsBuilderBlock)(id<PTDiffusionRecordTopicDetailsBuilder> builder);

/**
 The signature required for a builder block accepting a routing topic details builder.

 @deprecated Since 6.0. The use of PTDiffusionTopicDetails is being phased out
 in favor of PTDiffusionTopicSpecification. This interface will be removed in a
 future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
typedef void (^PTDiffusionRoutingTopicDetailsBuilderBlock)(id<PTDiffusionRoutingTopicDetailsBuilder> builder);

/**
 The signature required for a builder block accepting a single value topic details builder.
 
 @deprecated Since 6.0. The use of PTDiffusionTopicDetails is being phased out
 in favor of PTDiffusionTopicSpecification. This interface will be removed in a
 future release. Other single scalar topic types should now be used.
 See PTDiffusionPrimitive.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
typedef void (^PTDiffusionSingleValueTopicDetailsBuilderBlock)(id<PTDiffusionSingleValueTopicDetailsBuilder> builder);

/**
 The signature required for a builder block accepting a slave topic details builder.

 @deprecated Since 6.0. The use of PTDiffusionTopicDetails is being phased out
 in favor of PTDiffusionTopicSpecification. This interface will be removed in a
 future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
typedef void (^PTDiffusionSlaveTopicDetailsBuilderBlock)(id<PTDiffusionSlaveTopicDetailsBuilder> builder);

/**
 The signature required for a builder block accepting a basic topic details builder.

 @deprecated Since 6.0. The use of PTDiffusionTopicDetails is being phased out
 in favor of PTDiffusionTopicSpecification. This interface will be removed in a
 future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
typedef void (^PTDiffusionTopicDetailsBuilderBlock)(id<PTDiffusionTopicDetailsBuilder> builder);

/**
 @brief Topic details describe a topic.
 
 Where many topics are to have the same definition, the same topic detail can (and indeed should)
 be used to create many topics.

 Topic details are also used to provide details of existing topics to clients.
 In this case schema and/or attributes may not be present in the details if that level of detail
 was not requested.
 
 @see PTDiffusionTopicsFeature
 @see PTDiffusionTopicControlFeature

 @deprecated Since 6.0. The use of PTDiffusionTopicDetails is being phased out
 in favor of PTDiffusionTopicSpecification. This interface will be removed in a
 future release.

 @since 5.6
 */
__deprecated_msg("Will be removed in a future release.")
@interface PTDiffusionTopicDetails : NSObject <NSCopying>

+(instancetype)new NS_UNAVAILABLE;

-(instancetype)init NS_UNAVAILABLE;

/**
 Returns new topic details of the given type set with all default settings.
 
 This provides a simpler mechanism than using a builder where all of the builder's default
 values are suitable and there are no mandatory properties.

 @param type The topic type for which default details are to be returned.

 @exception PTDiffusionUnsupportedTopicDetailsBuildException If the creation of topic details
 for the given type is not supported by this client library.

 @exception PTDiffusionInvalidTopicDetailsBuilderStateException If default details can't be
 created for the specified type as a builder has to be used.
 
 @since 5.7
 */
+(instancetype)topicDetailsWithType:(PTDiffusionTopicType)type;

/**
 Returns new topic details for a record topic with all default settings.

 @since 5.7
 */
+(instancetype)recordTopicDetails;

/**
 Returns new topic details for a record topic configured using the given builder block which
 is supplied an object conforming to the PTDiffusionRecordTopicDetailsBuilder protocol.

 @since 5.7
 */
+(instancetype)recordTopicDetailsWithBuilderBlock:(PTDiffusionRecordTopicDetailsBuilderBlock)block;

/**
 Returns new topic details for a routing topic with all default settings.

 @since 5.7
 */
+(instancetype)routingTopicDetails;

/**
 Returns new topic details for a routing topic configured using the given builder block which
 is supplied an object conforming to the PTDiffusionRoutingTopicDetailsBuilder protocol.

 @since 5.7
 */
+(instancetype)routingTopicDetailsWithBuilderBlock:(PTDiffusionRoutingTopicDetailsBuilderBlock)block;

/**
 Returns new topic details for a single value topic with all default settings.
 
 @since 5.7
 */
+(instancetype)singleValueTopicDetails;

/**
 Returns new topic details for a single value topic configured using the given builder block which
 is supplied an object conforming to the PTDiffusionSingleValueTopicDetailsBuilder protocol.

 @since 5.7
 */
+(instancetype)singleValueTopicDetailsWithBuilderBlock:(PTDiffusionSingleValueTopicDetailsBuilderBlock)block;

/**
 Returns new topic details for a slave topic configured using the given builder block which
 is supplied an object conforming to the PTDiffusionSlaveTopicDetailsBuilder protocol.

 @since 5.7
 */
+(instancetype)slaveTopicDetailsWithBuilderBlock:(PTDiffusionSlaveTopicDetailsBuilderBlock)block;

/**
 Returns new topic details for a stateless topic with all default settings.
 
 @since 5.7
 */
+(instancetype)statelessTopicDetails;

/**
 Returns new topic details for a stateless topic configured using the given builder block which
 is supplied an object conforming to the PTDiffusionTopicDetailsBuilder protocol.

 @since 5.7
 */
+(instancetype)statelessTopicDetailsWithBuilderBlock:(PTDiffusionTopicDetailsBuilderBlock)block;

/**
 Attributes associated with this topic.
 
 @since 5.7
 */
@property(nullable, nonatomic, readonly) PTDiffusionTopicAttributes* attributes;

/**
 Schema associated with this topic.
 
 @since 5.7
 */
@property(nullable, nonatomic, readonly) PTDiffusionTopicSchema* schema;

/**
 The type of topic.

 @since 5.6
 */
@property(nonatomic, readonly) PTDiffusionTopicType type;

/**
 Compares the receiver to the given topic details.

 @param topicDetails The topic details object with which to compare the receiver.

 @return `YES` if the topic details is equal to the receiver, otherwise `NO`.
 
 @since 5.7
 */
-(BOOL)isEqualToTopicDetails:(nullable PTDiffusionTopicDetails *)topicDetails;

@end

/**
 @brief All topic details builders conform to this protocol.
 
 @see PTDiffusionTopicDetails

 @deprecated Since 6.0. The use of PTDiffusionTopicDetails is being phased out
 in favor of PTDiffusionTopicSpecification. This interface will be removed in a
 future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
@protocol PTDiffusionTopicDetailsBuilder <NSObject>

/**
 The topic reference or a zero length string if not set.

 @exception NSInvalidArgumentException On setting the property,
 if the supplied reference is `nil`.

 @since 5.7
 */
@property(nonatomic, copy) NSString* reference;

/**
 The 'tidy on unsubscribe' flag.

 By default, if a client unsubscribes from a topic, it may still receive
 content already queued for it on that topic.

 If this option is set to `YES`, when a client unsubscribes from a topic then
 any content for that topic that is still queued for the client is removed.
 There can be a performance overhead to using this option as the client queue
 must be locked whilst content is removed, however it may prove useful for
 preventing unwanted data being sent to clients.

 @since 5.7
 */
@property(nonatomic) BOOL tidiesOnUnsubscribe;

/**
 Sets a topic property. This is used for setting restricted or legacy topic properties.

 @param value The property value.

 @param key The property key. The values for these keys and permitted valuers are
 published in the user documentation.

 @exception NSInvalidArgumentException If either value or key is `nil`, or the
 key is not supported.

 @note Properties may be removed with the removePropertyForKey: method.

 @since 5.7
 */
-(void)setPropertyValue:(NSString *)value forKey:(NSString *)key;

/**
 Removes (unsets) a topic property.

 @param key The property key.

 @return `YES` if the key was set and has now been unset.
 
 @exception NSInvalidArgumentException If key is `nil`.

 @since 5.7
 */
-(BOOL)removePropertyForKey:(NSString *)key;

/**
 Gets the value of a topic property.

 @param key The property key. The values for these keys and permitted valuers are
 published in the user documentation.

 @exception NSInvalidArgumentException If key is `nil`.

 @since 5.7
 */
-(nullable NSString *)propertyValueForKey:(NSString *)key;

/**
 An immutable snapshot of the topic properties.

 @since 5.7
 */
@property(nonatomic, readonly) NSDictionary<NSString *, NSString *> * properties;

@end

/**
 @brief The protocol to which record topic details builders conform.

 @see PTDiffusionTopicDetails

 @deprecated Since 6.0. The use of PTDiffusionTopicDetails is being phased out
 in favor of PTDiffusionTopicSpecification. This interface will be removed in a
 future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
@protocol PTDiffusionRecordTopicDetailsBuilder <PTDiffusionTopicDetailsBuilder>

/**
 The record content schema defining the layout of stored record content values
 for this topic.

 @exception NSInvalidArgumentException On setting this property with a value of `nil`.

 @since 5.7
 */
@property(nonatomic, copy) PTDiffusionRecordContentSchema* metadata;

/**
 The value defining empty field.

 @exception NSInvalidArgumentException On setting this property with a value of `nil`.

 @since 5.7
 */
@property(nonatomic, copy) NSString* emptyFieldValue;

@end

/**
 @brief The protocol to which single value topic details builders conform.

 @see PTDiffusionTopicDetails

 @deprecated Since 6.0. The use of PTDiffusionTopicDetails is being phased out
 in favor of PTDiffusionTopicSpecification. This interface will be removed in a
 future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
@protocol PTDiffusionSingleValueTopicDetailsBuilder <PTDiffusionTopicDetailsBuilder>

/**
 The field metadata defining the format of stored field values for this topic.
 
 @exception NSInvalidArgumentException On setting this property with a value of `nil`.

 @since 5.7
 */
@property(nonatomic, copy) PTDiffusionFieldMetadata* metadata;

@end

/**
 @brief The protocol to which routing topic details builders conform.

 @see PTDiffusionTopicDetails

 @deprecated Since 6.0. The use of PTDiffusionTopicDetails is being phased out
 in favor of PTDiffusionTopicSpecification. This interface will be removed in a
 future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
@protocol PTDiffusionRoutingTopicDetailsBuilder <PTDiffusionTopicDetailsBuilder>

/**
 Sets the routing handler class name if routing is to be performed at the server.

 If routing is not to be handled at the server, it is expected that all subscription
 requests to the topic will be handled through a client using the subscription
 control feature.

 If not explicitly set, the value will be a zero length string indicating that
 routing is not handled by the server.
 
 @exception NSInvalidArgumentException On setting this property with a value of `nil`.

 @since 5.7
 */
@property(nonatomic, copy) NSString* handler;

@end

/**
 @brief The protocol to which slave topic details builders conform.

 @see PTDiffusionTopicDetails

 @deprecated Since 6.0. The use of PTDiffusionTopicDetails is being phased out
 in favor of PTDiffusionTopicSpecification. This interface will be removed in a
 future release.

 @since 5.7
 */
__deprecated_msg("Will be removed in a future release.")
@protocol PTDiffusionSlaveTopicDetailsBuilder <PTDiffusionTopicDetailsBuilder>

/**
 The master topic path.

 See PTDiffusionTopicType_Slave for full details of how slave topics operate.

 @exception NSInvalidArgumentException On setting this property with a value of `nil`.

 @since 5.7
 */
@property(nonatomic, copy) NSString* masterTopicPath;

@end

NS_ASSUME_NONNULL_END
